﻿using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;
using PdfSharp;
using PdfSharp.Pdf;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace PrintPreview
{
    public partial class Sample : Form
    {
        public Sample()
        {
            InitializeComponent();
        }

        #region Add PrinterLayer Elements

        private void Sample_Load(object sender, EventArgs e)
        {
            SetupMapWithBlankPage();
            AddMapLayer();
            AddScaleLineLayer();
            SetupComboBoxes();

            winformsMap1.Refresh();
        }

        private void SetupMapWithBlankPage()
        {
            // Setup the map unit, you want to always use feet or meters for printer layout maps
            winformsMap1.MapUnit = GeographyUnit.Meter;

            // Here we create the default zoom levels for the sample.  We have pre-created a PrinterZoomLevelSet
            // That pre-defines commonly used zoom levels based on percentages of zoom
            winformsMap1.ZoomLevelSet = new PrinterZoomLevelSet(winformsMap1.MapUnit,
                PrinterHelper.GetPointsPerGeographyUnit(winformsMap1.MapUnit));

            // Here we set the background color to gray so there is contrast with the while page
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.LightGray);

            // Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
            // The interactive overlay allows the layers to be interacted with
            PrinterInteractiveOverLay printerOverlay = new PrinterInteractiveOverLay();
            winformsMap1.InteractiveOverlays.Add("PrintPreviewOverlay", printerOverlay);
            winformsMap1.InteractiveOverlays.MoveToBottom("PrintPreviewOverlay");

            // Create the PagePrinterLayer which shows the page boundary and is the area the user will
            // arrange all of the other layer on top of.
            PagePrinterLayer pagePrinterLayer = new PagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait);
            pagePrinterLayer.Open();
            printerOverlay.PrinterLayers.Add("PageLayer", pagePrinterLayer);

            // Get the pages extent, slightly scale it up, and then set that as the default current extent
            winformsMap1.CurrentExtent = RectangleShape.ScaleUp(pagePrinterLayer.GetPosition(), 10).GetBoundingBox();

            // Set the minimum scale of the map to the last zoom level
            winformsMap1.MinimumScale = winformsMap1.ZoomLevelSet.ZoomLevel20.Scale;
        }

        private void AddMapLayer()
        {
            // Create the MapPrinterLayer and set the position
            MapPrinterLayer mapPrinterLayer = new MapPrinterLayer();
            mapPrinterLayer.MapUnit = GeographyUnit.Meter;
            mapPrinterLayer.BackgroundMask = new AreaStyle(new GeoPen(GeoColor.StandardColors.Black, 1));
            mapPrinterLayer.Open();

            // Set the maps position slightly below the pages center and 8 inches wide and 7 inches tall
            RectangleShape pageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch);
            mapPrinterLayer.SetPosition(8, 7, pageBoundingbox.GetCenterPoint().X, pageBoundingbox.GetCenterPoint().Y + 1,
                PrintingUnit.Inch);

            // Setup the Countries mapping layer                
            ShapeFileFeatureLayer shapefileFeatureLayer = new ShapeFileFeatureLayer(@"data/quadrat.shp",
                ShapeFileReadWriteMode.ReadOnly);
            shapefileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            // Add the class break style to the layer
            shapefileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle =
                AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.SimpleColors.Red, 2);

            // Add the new country layer to the MapPrinterLayer
            mapPrinterLayer.Layers.Add(shapefileFeatureLayer);

            // inMemoryFeaturLayer.Open();
            shapefileFeatureLayer.Open();
            mapPrinterLayer.MapExtent = shapefileFeatureLayer.GetBoundingBox();

            // Add the MapPrinterLayer to the PrinterInteractiveOverlay
            PrinterInteractiveOverLay printerInteractiveOverLay =
                (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            printerInteractiveOverLay.PrinterLayers.Add("MapLayer", mapPrinterLayer);
        }

        private void AddScaleLineLayer()
        {
            // Get the PrinterInteractiveOverlay
            PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];

            // Grab the MapPrinterLayer as we need to pass that into the scaleLinePrinterLayer so it
            // can be synced up to the map itself in real-time
            MapPrinterLayer mapPrinterLayer = (MapPrinterLayer)printerInteractiveOverLay.PrinterLayers["MapLayer"];

            //Create the scaleLinePrinterLayer and pass in the MapPrinterLayer
            ScaleLinePrinterLayer scaleLinePrinterLayer = new ScaleLinePrinterLayer(mapPrinterLayer);
            scaleLinePrinterLayer.MapUnit = GeographyUnit.DecimalDegree;

            // Set the scale line position offset from the page center and 1.25 inches wide and .25 inches tall
            RectangleShape pageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch);
            scaleLinePrinterLayer.SetPosition(1.25, .25, pageBoundingbox.GetCenterPoint().X - 3.25, pageBoundingbox.GetCenterPoint().Y - 2.25, PrintingUnit.Inch);

            // Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
            printerInteractiveOverLay.PrinterLayers.Add("ScaleLineLayer", scaleLinePrinterLayer);
        }

        #endregion

        #region Export Buttons

        private void btnPrintPreview_Click(object sender, EventArgs e)
        {
            PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            PagePrinterLayer pagePrinterLayer = printerInteractiveOverLay.PrinterLayers["PageLayer"] as PagePrinterLayer;

            PrintDocument printDocument = new PrintDocument();
            printDocument.DefaultPageSettings.Landscape = true;
            if (pagePrinterLayer.Orientation == PrinterOrientation.Portrait)
            {
                printDocument.DefaultPageSettings.Landscape = false;
            }

            printDocument.DefaultPageSettings.PaperSize = GetPrintPreviewPaperSize(pagePrinterLayer);

            PrinterGeoCanvas printerGeoCanvas = new PrinterGeoCanvas();
            printerGeoCanvas.DrawingArea = printDocument.DefaultPageSettings.Bounds;
            printerGeoCanvas.BeginDrawing(printDocument, pagePrinterLayer.GetBoundingBox(), winformsMap1.MapUnit);

            // Loop through all of the PrintingLayer in the PrinterInteractiveOverlay and print all of the
            // except for the PagePrinterLayer                
            Collection<SimpleCandidate> labelsInAllLayers = new Collection<SimpleCandidate>();
            foreach (PrinterLayer printerLayer in printerInteractiveOverLay.PrinterLayers)
            {
                printerLayer.IsDrawing = true;
                if (!(printerLayer is PagePrinterLayer))
                {
                    printerLayer.Draw(printerGeoCanvas, labelsInAllLayers);
                }
                printerLayer.IsDrawing = false;
            }

            printerGeoCanvas.Flush();

            PrintPreviewDialog printPreviewDialog = new PrintPreviewDialog();
            printPreviewDialog.Document = printDocument;
        }

        private void btnExportBitmap_Click(object sender, EventArgs e)
        {
            PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            PagePrinterLayer pagePrinterLayer = printerInteractiveOverLay.PrinterLayers["PageLayer"] as PagePrinterLayer;

            // Create a bitmap that is the size of the pages bounding box.  The page by default is in points unit
            // system which is very similar to pixels so that ampping is nice.
            Bitmap bitmap = null;
            try
            {
                bitmap = new Bitmap((int)pagePrinterLayer.GetBoundingBox().Width, (int)pagePrinterLayer.GetBoundingBox().Height);

                // Create a GdiPlusGeoCanvas and start the drawing
                GdiPlusGeoCanvas gdiPlusGeoCanvas = new GdiPlusGeoCanvas();
                gdiPlusGeoCanvas.BeginDrawing(bitmap, pagePrinterLayer.GetBoundingBox(), winformsMap1.MapUnit);

                // Loop through all of the PrintingLayer in the PrinterInteractiveOverlay and print all of the
                // except for the PagePrinterLayer                
                Collection<SimpleCandidate> labelsInAllLayers = new Collection<SimpleCandidate>();
                foreach (PrinterLayer printerLayer in printerInteractiveOverLay.PrinterLayers)
                {
                    printerLayer.IsDrawing = true;
                    if (!(printerLayer is PagePrinterLayer))
                    {
                        printerLayer.Draw(gdiPlusGeoCanvas, labelsInAllLayers);
                    }
                    printerLayer.IsDrawing = false;
                }

                // End the drawing
                gdiPlusGeoCanvas.EndDrawing();

                //Save the resulting bitmap to a file and open the file to show the user
                string filename = "PrintingResults.bmp";
                bitmap.Save(filename);
                Process.Start(filename);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " " + ex.StackTrace, "Exception");
            }
            finally
            {
                if (bitmap != null) { bitmap.Dispose(); }
            }
        }

        private void btnExportToPdf_Click(object sender, EventArgs e)
        {
            // Get the PrinterInteractiveOverlay and PagePrinterLayer as we are going to need them below
            PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            PagePrinterLayer pagePrinterLayer = printerInteractiveOverLay.PrinterLayers["PageLayer"] as PagePrinterLayer;

            PdfDocument pdfDocument = null;
            PdfPage pdfPage = null;

            try
            {
                // Create the PDF documents, pages, and copnfigure them
                pdfDocument = new PdfDocument();
                pdfPage = pdfDocument.AddPage();
                pdfPage.Orientation = pagePrinterLayer.Orientation == PrinterOrientation.Portrait ? PageOrientation.Portrait : PageOrientation.Landscape;
                pdfPage.Size = GetPdfPageSize(pagePrinterLayer.PageSize);

                // Create the PdfGeoCanvas and pass tha PdfPage into the BeginDrawing 
                PdfGeoCanvas pdfGeoCanvas = new PdfGeoCanvas();

                pdfGeoCanvas.BeginDrawing(pdfPage, pagePrinterLayer.GetBoundingBox(), winformsMap1.MapUnit);


                MapPrinterLayer mapPrinterLayer = (MapPrinterLayer)printerInteractiveOverLay.PrinterLayers["MapLayer"];

                var currentBoundingBox = mapPrinterLayer.GetBoundingBox();
                RectangleShape adjustedWorldExtent = ExtentHelper.GetDrawingExtent(mapPrinterLayer.MapExtent, (float)currentBoundingBox.Width, (float)currentBoundingBox.Height);

                mapPrinterLayer.MapExtent = ExtentHelper.ZoomToScale(10000, adjustedWorldExtent, GeographyUnit.Meter, (float)currentBoundingBox.Width, (float)currentBoundingBox.Height);
                // Loop through all of the PrinterLayers and draw them to the canvas except for the
                // PagePrinterLayer which is just used for the visual layout
                Collection<SimpleCandidate> labelsInAllLayers = new Collection<SimpleCandidate>();
                foreach (PrinterLayer printerLayer in printerInteractiveOverLay.PrinterLayers)
                {
                    printerLayer.IsDrawing = true;
                    if (!(printerLayer is PagePrinterLayer))
                    {
                        printerLayer.Draw(pdfGeoCanvas, labelsInAllLayers);
                    }
                    printerLayer.IsDrawing = false;
                }

                // Finish up the drawing
                pdfGeoCanvas.EndDrawing();

                // Dave the PDF document and launch it in the default viewer to show the user
                string filename = "PrintingResults.pdf";
                pdfDocument.Save(filename);
                Process.Start(filename);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " " + ex.StackTrace, "Exception");
            }
            finally
            {
                if (pdfDocument != null) { pdfDocument.Dispose(); }
            }
        }

        private void btnPrint_Click(object sender, EventArgs e)
        {
            PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            PagePrinterLayer pagePrinterLayer = printerInteractiveOverLay.PrinterLayers["PageLayer"] as PagePrinterLayer;

            PrintDocument printDocument = new PrintDocument();
            printDocument.DefaultPageSettings.Landscape = true;
            if (pagePrinterLayer.Orientation == PrinterOrientation.Portrait)
            {
                printDocument.DefaultPageSettings.Landscape = false;
            }

            printDocument.DefaultPageSettings.PaperSize = GetPrintPreviewPaperSize(pagePrinterLayer);

            PrinterGeoCanvas printerGeoCanvas = new PrinterGeoCanvas();
            printerGeoCanvas.DrawingArea = new Rectangle(0, 0, Convert.ToInt32(printDocument.DefaultPageSettings.PrintableArea.Width), Convert.ToInt32(printDocument.DefaultPageSettings.PrintableArea.Height));
            printerGeoCanvas.BeginDrawing(printDocument, pagePrinterLayer.GetBoundingBox(), winformsMap1.MapUnit);

            // Loop through all of the PrintingLayer in the PrinterInteractiveOverlay and print all of the
            // except for the PagePrinterLayer                
            Collection<SimpleCandidate> labelsInAllLayers = new Collection<SimpleCandidate>();
            foreach (PrinterLayer printerLayer in printerInteractiveOverLay.PrinterLayers)
            {
                printerLayer.IsDrawing = true;
                if (!(printerLayer is PagePrinterLayer))
                {
                    printerLayer.Draw(printerGeoCanvas, labelsInAllLayers);
                }
                printerLayer.IsDrawing = false;
            }

            printerGeoCanvas.EndDrawing();
        }
        #endregion

        #region Map Events
        void winformsMap1_CurrentScaleChanged(object sender, CurrentScaleChangedWinformsMapEventArgs e)
        {
            // Here we sync up the zoom combox to the map's zoom level.
            PrinterZoomLevelSet printerZoomLevelSet = (PrinterZoomLevelSet)winformsMap1.ZoomLevelSet;
            ZoomLevel currentZoomLevel = printerZoomLevelSet.GetZoomLevel(winformsMap1.CurrentExtent, winformsMap1.Width, winformsMap1.MapUnit);
            cboZoom.SelectedItem = printerZoomLevelSet.GetZoomPercentage(currentZoomLevel) + "%";
        }

        private void winformsMap1_MapClick(object sender, MapClickWinformsMapEventArgs e)
        {
            // Here we loop through all of the PrinterLayers to find the one the user right mouse clicked on.
            // Then we show the right click menu to give the user some options
            if (e.MouseButton == MapMouseButton.Right)
            {
                PrinterInteractiveOverLay printerOverlay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
                for (int i = printerOverlay.PrinterLayers.Count - 1; i >= 0; i--)
                {
                    if (printerOverlay.PrinterLayers[i].GetType() != typeof(PagePrinterLayer))
                    {
                        RectangleShape boundingBox = printerOverlay.PrinterLayers[i].GetPosition();
                        if (boundingBox.Contains(e.WorldLocation))
                        {
                            cmRightClick.Tag = printerOverlay.PrinterLayers[i];
                            cmRightClick.Show(winformsMap1, new Point((int)(e.ScreenX), (int)(e.ScreenY)));
                            break;
                        }
                    }
                }
            }
        }
        #endregion

        #region Pan & Zoom Buttons

        private void btnZoomIn_Click(object sender, EventArgs e)
        {
            // Grab the MapPrinterLayer and adjust the extent
            PrinterInteractiveOverLay printerOverlay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            MapPrinterLayer mapPrinterLayer = ((MapPrinterLayer)(printerOverlay.PrinterLayers["MapLayer"]));

            // Here we snap the current scale to the default zoomLevelSet when we zoom in
            ZoomLevelSet zoomLevelSet = new ZoomLevelSet();
            double newScale = ZoomLevelSet.GetLowerZoomLevelScale(ExtentHelper.GetScale(mapPrinterLayer.MapExtent, (float)mapPrinterLayer.GetBoundingBox().Width, mapPrinterLayer.MapUnit), zoomLevelSet);
            mapPrinterLayer.MapExtent = ExtentHelper.ZoomToScale(newScale, mapPrinterLayer.MapExtent, mapPrinterLayer.MapUnit, (float)mapPrinterLayer.GetBoundingBox().Width, (float)mapPrinterLayer.GetBoundingBox().Height);

            winformsMap1.Refresh();
        }

        private void btnZoomOut_Click(object sender, EventArgs e)
        {
            // Grab the MapPrinterLayer and adjust the extent
            PrinterInteractiveOverLay printerOverlay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            MapPrinterLayer mapPrinterLayer = ((MapPrinterLayer)(printerOverlay.PrinterLayers["MapLayer"]));

            // Here we snap the current scale to the default zoomLevelSet when we zoom out
            ZoomLevelSet zoomLevelSet = new ZoomLevelSet();
            double newScale = ZoomLevelSet.GetHigherZoomLevelScale(ExtentHelper.GetScale(mapPrinterLayer.MapExtent, (float)mapPrinterLayer.GetBoundingBox().Width, mapPrinterLayer.MapUnit), zoomLevelSet);
            mapPrinterLayer.MapExtent = ExtentHelper.ZoomToScale(newScale, mapPrinterLayer.MapExtent, mapPrinterLayer.MapUnit, (float)mapPrinterLayer.GetBoundingBox().Width, (float)mapPrinterLayer.GetBoundingBox().Height);

            winformsMap1.Refresh();
        }

        private void btnPan_Click(object sender, EventArgs e)
        {
            // Grab the MapPrinterLayer and adjust the extent
            PrinterInteractiveOverLay printerOverlay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];

            MapPrinterLayer mapPrinterLayer = ((MapPrinterLayer)(printerOverlay.PrinterLayers["MapLayer"]));

            if (sender == btnPanUp)
            { mapPrinterLayer.MapExtent = ExtentHelper.Pan(mapPrinterLayer.MapExtent, PanDirection.Up, 30); }
            else if (sender == btnPanDown)
            { mapPrinterLayer.MapExtent = ExtentHelper.Pan(mapPrinterLayer.MapExtent, PanDirection.Down, 30); }
            else if (sender == btnPanLeft)
            { mapPrinterLayer.MapExtent = ExtentHelper.Pan(mapPrinterLayer.MapExtent, PanDirection.Left, 30); }
            else if (sender == btnPanRight)
            { mapPrinterLayer.MapExtent = ExtentHelper.Pan(mapPrinterLayer.MapExtent, PanDirection.Right, 30); }

            winformsMap1.Refresh();
        }

        #endregion

        #region Combo Boxes

        private void SetupComboBoxes()
        {
            // Set the default combobox items
            cboPageSize.SelectedIndex = 0;
            cboOrientation.SelectedIndex = 0;

            //Setup the zoom level combobox to relfect the zoom percentages
            foreach (ZoomLevel level in winformsMap1.ZoomLevelSet.GetZoomLevels())
            {
                cboZoom.Items.Add(((PrinterZoomLevelSet)winformsMap1.ZoomLevelSet).GetZoomPercentage(level) + "%");
            }
        }

        private void cboPageSize_SelectedIndexChanged(object sender, EventArgs e)
        {
            // When the page size combobox is changed then we update the PagePrinterLayer to reflect the changes
            if (((ComboBox)sender).Focused)
            {
                PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
                PagePrinterLayer pagePrinterLayer = (PagePrinterLayer)printerInteractiveOverLay.PrinterLayers["PageLayer"];

                string selectText = cboPageSize.SelectedItem.ToString();
                if (selectText.StartsWith("ANSI A"))
                    pagePrinterLayer.PageSize = PrinterPageSize.AnsiA;
                else if (selectText.StartsWith("ANSI B"))
                    pagePrinterLayer.PageSize = PrinterPageSize.AnsiB;
                else if (selectText.StartsWith("ANSI C"))
                    pagePrinterLayer.PageSize = PrinterPageSize.AnsiC;
                else if (selectText.StartsWith("ANSI D"))
                    pagePrinterLayer.PageSize = PrinterPageSize.AnsiD;
                else if (selectText.StartsWith("ANSI E"))
                    pagePrinterLayer.PageSize = PrinterPageSize.AnsiE;

                winformsMap1.Refresh();
            }
        }

        private void cboOrientation_SelectedIndexChanged(object sender, EventArgs e)
        {
            // When the orientation combobox is changed then we update the PagePrinterLayer to reflect the changes
            if (((ComboBox)sender).Focused)
            {
                PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
                PagePrinterLayer pagePrinterLayer = (PagePrinterLayer)printerInteractiveOverLay.PrinterLayers["PageLayer"];

                string selectText = cboOrientation.SelectedItem.ToString();
                if (selectText.ToUpper() == "LANDSCAPE")
                {
                    pagePrinterLayer.Orientation = PrinterOrientation.Landscape;
                }
                else if (selectText.ToUpper() == "PORTRAIT")
                {
                    pagePrinterLayer.Orientation = PrinterOrientation.Portrait;
                }

                winformsMap1.Refresh();
            }
        }

        private void cboZoom_SelectionChangeCommitted(object sender, EventArgs e)
        {
            if (((ComboBox)sender).Focused)
            {
                PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
                MapPrinterLayer pagePrinterLayer = (MapPrinterLayer)printerInteractiveOverLay.PrinterLayers["MapLayer"];
                String zoomString = (String)cboZoom.SelectedItem;
                double currentZoom = Double.Parse(zoomString.Replace("%", ""));
                winformsMap1.CurrentScale = PrinterHelper.GetPointsPerGeographyUnit(winformsMap1.MapUnit) / (currentZoom / 100);
                winformsMap1.Refresh();
            }
        }

        #endregion

        #region Toolbox
        private void toolStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            PrinterLayer printerLayer = null;

            string name = e.ClickedItem.Text.ToLower();
            switch (name)
            {
                case "label":
                    printerLayer = new LabelPrinterLayer();
                    printerLayer.SetPosition(2, 1, 0, 0, PrintingUnit.Inch);
                    break;
                case "image":
                    printerLayer = new ImagePrinterLayer();
                    break;
                case "scale line":
                    MessageBox.Show("NotImplemented");
                    break;
                case "scale bar":
                    MessageBox.Show("NotImplemented");
                    break;
                case "data grid":
                    printerLayer = new DataGridPrinterLayer();
                    printerLayer.SetPosition(1, 1, 0, 0, PrintingUnit.Inch);
                    break;
                default:
                    break;
            }

            if (printerLayer != null && ShowPrinterLayerProperties(printerLayer) == DialogResult.OK)
            {
                printerInteractiveOverLay.PrinterLayers.Add(printerLayer);
                winformsMap1.Refresh();
            }
        }
        #endregion

        #region Right Click and Dialogs
        private void cmRightClick_Click(object sender, EventArgs e)
        {
            PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            PrinterLayer printerLayer = (PrinterLayer)cmRightClick.Tag;

            ToolStripItem selectedItem = cmRightClick.GetItemAt(((MouseEventArgs)e).X, ((MouseEventArgs)e).Y);
            if (selectedItem.Name == "tsmRemove")
            {
                printerInteractiveOverLay.PrinterLayers.Remove(printerLayer);
            }
            else if (selectedItem.Name == "tsmProperty")
            {
                ShowPrinterLayerProperties(printerLayer);
            }
            winformsMap1.Refresh();
        }

        private DialogResult ShowPrinterLayerProperties(PrinterLayer printerLayer)
        {
            DialogResult dialogResult = DialogResult.Cancel;

            if (printerLayer.GetType() == typeof(LabelPrinterLayer))
            {
                LabelPrinterLayerProperty dialog = new LabelPrinterLayerProperty((LabelPrinterLayer)printerLayer);
                dialog.StartPosition = FormStartPosition.CenterParent;
                dialogResult = dialog.ShowDialog();
            }
            else if (printerLayer.GetType() == typeof(ScaleBarPrinterLayer))
            {
                MessageBox.Show("Scalebar (Coming Soon)");
            }
            else if (printerLayer.GetType() == typeof(ScaleLinePrinterLayer))
            {
                MessageBox.Show("Scaleline (Coming Soon)");
            }
            else if (printerLayer.GetType() == typeof(ImagePrinterLayer))
            {
                ImagePrinterLayerProperty dialog = new ImagePrinterLayerProperty((ImagePrinterLayer)printerLayer);
                dialog.StartPosition = FormStartPosition.CenterParent;
                dialogResult = dialog.ShowDialog();
            }
            else if (printerLayer.GetType() == typeof(DataGridPrinterLayer))
            {
                DataGridPrinterLayerProperty dialog = new DataGridPrinterLayerProperty((DataGridPrinterLayer)printerLayer);
                dialog.StartPosition = FormStartPosition.CenterParent;
                dialogResult = dialog.ShowDialog();
            }

            return dialogResult;
        }
        #endregion

        #region Helper Methods

        private RectangleShape GetPageBoundingBox(PrintingUnit unit)
        {
            // This helper method gets the pages bounding box in the unit requested
            PrinterInteractiveOverLay printerInteractiveOverLay = (PrinterInteractiveOverLay)winformsMap1.InteractiveOverlays["PrintPreviewOverlay"];
            PagePrinterLayer pagePrinterLayer = (PagePrinterLayer)printerInteractiveOverLay.PrinterLayers["PageLayer"];
            return pagePrinterLayer.GetPosition(unit); ;
        }

        private PageSize GetPdfPageSize(PrinterPageSize pageSize)
        {
            PageSize pdfPageSize = PageSize.Letter;
            switch (pageSize)
            {
                case PrinterPageSize.AnsiA:
                    pdfPageSize = PageSize.Letter;
                    break;
                case PrinterPageSize.AnsiB:
                    pdfPageSize = PageSize.Ledger;
                    break;
                case PrinterPageSize.AnsiC:
                    pdfPageSize = PageSize.A2;
                    break;
                case PrinterPageSize.AnsiD:
                    pdfPageSize = PageSize.A1;
                    break;
                case PrinterPageSize.AnsiE:
                    pdfPageSize = PageSize.A0;
                    break;
                case PrinterPageSize.Custom:
                    throw new NotSupportedException();
                default:
                    throw new NotSupportedException();
            }
            return pdfPageSize;
        }

        private PaperSize GetPrintPreviewPaperSize(PagePrinterLayer pagePrinterLayer)
        {
            PaperSize printPreviewPaperSize = new PaperSize("AnsiA", 850, 1100);
            switch (pagePrinterLayer.PageSize)
            {
                case PrinterPageSize.AnsiA:
                    printPreviewPaperSize = new PaperSize("AnsiA", 850, 1100);
                    break;
                case PrinterPageSize.AnsiB:
                    printPreviewPaperSize = new PaperSize("AnsiB", 1100, 1700);
                    break;
                case PrinterPageSize.AnsiC:
                    printPreviewPaperSize = new PaperSize("AnsiC", 1700, 2200);
                    break;
                case PrinterPageSize.AnsiD:
                    printPreviewPaperSize = new PaperSize("AnsiD", 2200, 3400);
                    break;
                case PrinterPageSize.AnsiE:
                    printPreviewPaperSize = new PaperSize("AnsiE", 3400, 4400);
                    break;
                case PrinterPageSize.Custom:
                    printPreviewPaperSize = new PaperSize("Custom Size", (int)pagePrinterLayer.CustomWidth, (int)pagePrinterLayer.CustomHeight);
                    break;
                default:
                    break;
            }

            return printPreviewPaperSize;
        }

        #endregion
    }
}